@@ -35,15 +35,7 @@ if TYPE_CHECKING:
     from narwhals._spark_like.dataframe import SparkLikeLazyFrame
     from narwhals._spark_like.namespace import SparkLikeNamespace
     from narwhals._utils import _LimitedContext
-    from narwhals.typing import (
-        FillNullStrategy,
-        IntoDType,
-        NonNestedLiteral,
-        NumericLiteral,
-        PythonLiteral,
-        RankMethod,
-        TemporalLiteral,
-    )
+    from narwhals.typing import FillNullStrategy, IntoDType, NonNestedLiteral, RankMethod
 
     NativeRankMethod: TypeAlias = Literal["rank", "dense_rank", "row_number"]
     SparkWindowFunction = WindowFunction[SparkLikeLazyFrame, Column]
@@ -77,18 +69,9 @@ class SparkLikeExpr(SQLExpr["SparkLikeLazyFrame", "Column"]):
         "ordinal": "row_number",
     }
 
-    def _function(self, name: str, *args: Column | PythonLiteral) -> Column:
-        return getattr(self._F, name)(*args)
-
-    def _lit(self, value: Any) -> Column:
-        return self._F.lit(value)
-
     def _count_star(self) -> Column:
         return self._F.count("*")
 
-    def _when(self, condition: Column, value: Column) -> Column:
-        return self._F.when(condition, value)
-
     def _window_expression(
         self,
         expr: Column,
@@ -243,44 +226,6 @@ class SparkLikeExpr(SQLExpr["SparkLikeLazyFrame", "Column"]):
         invert = cast("Callable[..., Column]", operator.invert)
         return self._with_elementwise(invert)
 
-    def all(self) -> Self:
-        def f(expr: Column) -> Column:
-            return self._F.coalesce(self._F.bool_and(expr), self._F.lit(True))  # noqa: FBT003
-
-        def window_f(
-            df: SparkLikeLazyFrame, window_inputs: SparkWindowInputs
-        ) -> Sequence[Column]:
-            return [
-                self._F.coalesce(
-                    self._F.bool_and(expr).over(
-                        self.partition_by(*window_inputs.partition_by)
-                    ),
-                    self._F.lit(True),  # noqa: FBT003
-                )
-                for expr in self(df)
-            ]
-
-        return self._with_callable(f)._with_window_function(window_f)
-
-    def any(self) -> Self:
-        def f(expr: Column) -> Column:
-            return self._F.coalesce(self._F.bool_or(expr), self._F.lit(False))  # noqa: FBT003
-
-        def window_f(
-            df: SparkLikeLazyFrame, window_inputs: SparkWindowInputs
-        ) -> Sequence[Column]:
-            return [
-                self._F.coalesce(
-                    self._F.bool_or(expr).over(
-                        self.partition_by(*window_inputs.partition_by)
-                    ),
-                    self._F.lit(False),  # noqa: FBT003
-                )
-                for expr in self(df)
-            ]
-
-        return self._with_callable(f)._with_window_function(window_f)
-
     def cast(self, dtype: IntoDType) -> Self:
         def func(df: SparkLikeLazyFrame) -> Sequence[Column]:
             spark_dtype = narwhals_to_native_dtype(
@@ -305,9 +250,6 @@ class SparkLikeExpr(SQLExpr["SparkLikeLazyFrame", "Column"]):
             implementation=self._implementation,
         )
 
-    def count(self) -> Self:
-        return self._with_callable(self._F.count)
-
     def median(self) -> Self:
         def _median(expr: Column) -> Column:
             if self._implementation in {
@@ -327,25 +269,6 @@ class SparkLikeExpr(SQLExpr["SparkLikeLazyFrame", "Column"]):
 
         return self._with_callable(_null_count)
 
-    def sum(self) -> Self:
-        def f(expr: Column) -> Column:
-            return self._F.coalesce(self._F.sum(expr), self._F.lit(0))
-
-        def window_f(
-            df: SparkLikeLazyFrame, window_inputs: SparkWindowInputs
-        ) -> Sequence[Column]:
-            return [
-                self._F.coalesce(
-                    self._F.sum(expr).over(
-                        self.partition_by(*window_inputs.partition_by)
-                    ),
-                    self._F.lit(0),
-                )
-                for expr in self(df)
-            ]
-
-        return self._with_callable(f)._with_window_function(window_f)
-
     def std(self, ddof: int) -> Self:
         F = self._F  # noqa: N806
         if ddof == 0:
@@ -372,34 +295,6 @@ class SparkLikeExpr(SQLExpr["SparkLikeLazyFrame", "Column"]):
 
         return self._with_callable(func)
 
-    def clip(
-        self,
-        lower_bound: Self | NumericLiteral | TemporalLiteral | None = None,
-        upper_bound: Self | NumericLiteral | TemporalLiteral | None = None,
-    ) -> Self:
-        def _clip_lower(expr: Column, lower_bound: Column) -> Column:
-            result = expr
-            return self._F.when(result < lower_bound, lower_bound).otherwise(result)
-
-        def _clip_upper(expr: Column, upper_bound: Column) -> Column:
-            result = expr
-            return self._F.when(result > upper_bound, upper_bound).otherwise(result)
-
-        def _clip_both(expr: Column, lower_bound: Column, upper_bound: Column) -> Column:
-            return (
-                self._F.when(expr < lower_bound, lower_bound)
-                .when(expr > upper_bound, upper_bound)
-                .otherwise(expr)
-            )
-
-        if lower_bound is None:
-            return self._with_elementwise(_clip_upper, upper_bound=upper_bound)
-        if upper_bound is None:
-            return self._with_elementwise(_clip_lower, lower_bound=lower_bound)
-        return self._with_elementwise(
-            _clip_both, lower_bound=lower_bound, upper_bound=upper_bound
-        )
-
     def is_finite(self) -> Self:
         def _is_finite(expr: Column) -> Column:
             # A value is finite if it's not NaN, and not infinite, while NULLs should be
@@ -492,12 +387,6 @@ class SparkLikeExpr(SQLExpr["SparkLikeLazyFrame", "Column"]):
 
         return self._with_elementwise(_log)
 
-    def exp(self) -> Self:
-        def _exp(expr: Column) -> Column:
-            return self._F.exp(expr)
-
-        return self._with_elementwise(_exp)
-
     def sqrt(self) -> Self:
         def _sqrt(expr: Column) -> Column:
             return self._F.when(expr < 0, self._F.lit(float("nan"))).otherwise(
