@@ -1,20 +1,20 @@
 from __future__ import annotations
 
+import enum
 from collections import OrderedDict
+from collections.abc import Iterable, Mapping
 from datetime import timezone
 from itertools import starmap
 from typing import TYPE_CHECKING
-from typing import Mapping
 
-from narwhals.utils import isinstance_or_issubclass
+from narwhals._utils import _DeferredIterable, isinstance_or_issubclass
 
 if TYPE_CHECKING:
-    from typing import Iterator
-    from typing import Sequence
+    from collections.abc import Iterator, Sequence
 
     from typing_extensions import Self
 
-    from narwhals.typing import TimeUnit
+    from narwhals.typing import IntoDType, TimeUnit
 
 
 def _validate_dtype(dtype: DType | type[DType]) -> None:
@@ -27,7 +27,7 @@ def _validate_dtype(dtype: DType | type[DType]) -> None:
 
 
 class DType:
-    def __repr__(self: Self) -> str:  # pragma: no cover
+    def __repr__(self) -> str:  # pragma: no cover
         return self.__class__.__qualname__
 
     @classmethod
@@ -62,12 +62,12 @@ class DType:
     def is_nested(cls: type[Self]) -> bool:
         return issubclass(cls, NestedType)
 
-    def __eq__(self: Self, other: DType | type[DType]) -> bool:  # type: ignore[override]
-        from narwhals.utils import isinstance_or_issubclass
+    def __eq__(self, other: DType | type[DType]) -> bool:  # type: ignore[override]
+        from narwhals._utils import isinstance_or_issubclass
 
         return isinstance_or_issubclass(other, type(self))
 
-    def __hash__(self: Self) -> int:
+    def __hash__(self) -> int:
         return hash(self.__class__)
 
 
@@ -112,27 +112,34 @@ class Decimal(NumericType):
 
 
 class Int128(SignedIntegerType):
-    """128-bit signed integer type."""
+    """128-bit signed integer type.
+
+    Examples:
+        >>> import polars as pl
+        >>> import pyarrow as pa
+        >>> import narwhals as nw
+        >>> import duckdb
+        >>> s_native = pl.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> df_native = pa.table({"a": [2, 1, 3, 7]})
+        >>> rel = duckdb.sql(" SELECT CAST (a AS INT128) AS a FROM df_native ")
+
+        >>> s.cast(nw.Int128).dtype
+        Int128
+        >>> nw.from_native(rel).collect_schema()["a"]
+        Int128
+    """
 
 
 class Int64(SignedIntegerType):
     """64-bit signed integer type.
 
     Examples:
-        >>> import pandas as pd
         >>> import polars as pl
-        >>> import pyarrow as pa
         >>> import narwhals as nw
-        >>> data = [2, 1, 3, 7]
-        >>> ser_pd = pd.Series(data)
-        >>> ser_pl = pl.Series(data)
-        >>> ser_pa = pa.chunked_array([data])
-
-        >>> nw.from_native(ser_pd, series_only=True).dtype
-        Int64
-        >>> nw.from_native(ser_pl, series_only=True).dtype
-        Int64
-        >>> nw.from_native(ser_pa, series_only=True).dtype
+        >>> s_native = pl.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.Int64).dtype
         Int64
     """
 
@@ -141,24 +148,11 @@ class Int32(SignedIntegerType):
     """32-bit signed integer type.
 
     Examples:
-        >>> import pandas as pd
-        >>> import polars as pl
         >>> import pyarrow as pa
         >>> import narwhals as nw
-        >>> data = [2, 1, 3, 7]
-        >>> ser_pd = pd.Series(data)
-        >>> ser_pl = pl.Series(data)
-        >>> ser_pa = pa.chunked_array([data])
-
-        >>> def func(ser):
-        ...     ser_nw = nw.from_native(ser, series_only=True)
-        ...     return ser_nw.cast(nw.Int32).dtype
-
-        >>> func(ser_pd)
-        Int32
-        >>> func(ser_pl)
-        Int32
-        >>> func(ser_pa)
+        >>> s_native = pa.chunked_array([[2, 1, 3, 7]])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.Int32).dtype
         Int32
     """
 
@@ -167,24 +161,11 @@ class Int16(SignedIntegerType):
     """16-bit signed integer type.
 
     Examples:
-        >>> import pandas as pd
         >>> import polars as pl
-        >>> import pyarrow as pa
         >>> import narwhals as nw
-        >>> data = [2, 1, 3, 7]
-        >>> ser_pd = pd.Series(data)
-        >>> ser_pl = pl.Series(data)
-        >>> ser_pa = pa.chunked_array([data])
-
-        >>> def func(ser):
-        ...     ser_nw = nw.from_native(ser, series_only=True)
-        ...     return ser_nw.cast(nw.Int16).dtype
-
-        >>> func(ser_pd)
-        Int16
-        >>> func(ser_pl)
-        Int16
-        >>> func(ser_pa)
+        >>> s_native = pl.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.Int16).dtype
         Int16
     """
 
@@ -193,55 +174,39 @@ class Int8(SignedIntegerType):
     """8-bit signed integer type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [2, 1, 3, 7]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> def func(ser):
-       ...     ser_nw = nw.from_native(ser, series_only=True)
-       ...     return ser_nw.cast(nw.Int8).dtype
-
-       >>> func(ser_pd)
-       Int8
-       >>> func(ser_pl)
-       Int8
-       >>> func(ser_pa)
-       Int8
+        >>> import pandas as pd
+        >>> import narwhals as nw
+        >>> s_native = pd.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.Int8).dtype
+        Int8
     """
 
 
 class UInt128(UnsignedIntegerType):
-    """128-bit unsigned integer type."""
+    """128-bit unsigned integer type.
+
+    Examples:
+        >>> import pandas as pd
+        >>> import narwhals as nw
+        >>> import duckdb
+        >>> df_native = pd.DataFrame({"a": [2, 1, 3, 7]})
+        >>> rel = duckdb.sql(" SELECT CAST (a AS UINT128) AS a FROM df_native ")
+        >>> nw.from_native(rel).collect_schema()["a"]
+        UInt128
+    """
 
 
 class UInt64(UnsignedIntegerType):
     """64-bit unsigned integer type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [2, 1, 3, 7]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> def func(ser):
-       ...     ser_nw = nw.from_native(ser, series_only=True)
-       ...     return ser_nw.cast(nw.UInt64).dtype
-
-       >>> func(ser_pd)
-       UInt64
-       >>> func(ser_pl)
-       UInt64
-       >>> func(ser_pa)
-       UInt64
+        >>> import pandas as pd
+        >>> import narwhals as nw
+        >>> s_native = pd.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.UInt64).dtype
+        UInt64
     """
 
 
@@ -249,25 +214,12 @@ class UInt32(UnsignedIntegerType):
     """32-bit unsigned integer type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [2, 1, 3, 7]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> def func(ser):
-       ...     ser_nw = nw.from_native(ser, series_only=True)
-       ...     return ser_nw.cast(nw.UInt32).dtype
-
-       >>> func(ser_pd)
-       UInt32
-       >>> func(ser_pl)
-       UInt32
-       >>> func(ser_pa)
-       UInt32
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> s_native = pl.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.UInt32).dtype
+        UInt32
     """
 
 
@@ -275,25 +227,12 @@ class UInt16(UnsignedIntegerType):
     """16-bit unsigned integer type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [2, 1, 3, 7]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> def func(ser):
-       ...     ser_nw = nw.from_native(ser, series_only=True)
-       ...     return ser_nw.cast(nw.UInt16).dtype
-
-       >>> func(ser_pd)
-       UInt16
-       >>> func(ser_pl)
-       UInt16
-       >>> func(ser_pa)
-       UInt16
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> s_native = pl.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.UInt16).dtype
+        UInt16
     """
 
 
@@ -301,25 +240,12 @@ class UInt8(UnsignedIntegerType):
     """8-bit unsigned integer type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [2, 1, 3, 7]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> def func(ser):
-       ...     ser_nw = nw.from_native(ser, series_only=True)
-       ...     return ser_nw.cast(nw.UInt8).dtype
-
-       >>> func(ser_pd)
-       UInt8
-       >>> func(ser_pl)
-       UInt8
-       >>> func(ser_pa)
-       UInt8
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> s_native = pl.Series([2, 1, 3, 7])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.UInt8).dtype
+        UInt8
     """
 
 
@@ -327,20 +253,11 @@ class Float64(FloatType):
     """64-bit floating point type.
 
     Examples:
-        >>> import pandas as pd
-        >>> import polars as pl
         >>> import pyarrow as pa
         >>> import narwhals as nw
-        >>> data = [0.001, 0.1, 0.01, 0.1]
-        >>> ser_pd = pd.Series(data)
-        >>> ser_pl = pl.Series(data)
-        >>> ser_pa = pa.chunked_array([data])
-
-        >>> nw.from_native(ser_pd, series_only=True).dtype
-        Float64
-        >>> nw.from_native(ser_pl, series_only=True).dtype
-        Float64
-        >>> nw.from_native(ser_pa, series_only=True).dtype
+        >>> s_native = pa.chunked_array([[0.001, 0.1, 0.01, 0.1]])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.Float64).dtype
         Float64
     """
 
@@ -349,25 +266,12 @@ class Float32(FloatType):
     """32-bit floating point type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [0.001, 0.1, 0.01, 0.1]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> def func(ser):
-       ...     ser_nw = nw.from_native(ser, series_only=True)
-       ...     return ser_nw.cast(nw.Float32).dtype
-
-       >>> func(ser_pd)
-       Float32
-       >>> func(ser_pl)
-       Float32
-       >>> func(ser_pa)
-       Float32
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> s_native = pl.Series([0.001, 0.1, 0.01, 0.1])
+        >>> s = nw.from_native(s_native, series_only=True)
+        >>> s.cast(nw.Float32).dtype
+        Float32
     """
 
 
@@ -375,21 +279,11 @@ class String(DType):
     """UTF-8 encoded string type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = ["beluga", "narwhal", "orca", "vaquita"]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> nw.from_native(ser_pd, series_only=True).dtype
-       String
-       >>> nw.from_native(ser_pl, series_only=True).dtype
-       String
-       >>> nw.from_native(ser_pa, series_only=True).dtype
-       String
+        >>> import pandas as pd
+        >>> import narwhals as nw
+        >>> s_native = pd.Series(["beluga", "narwhal", "orca", "vaquita"])
+        >>> nw.from_native(s_native, series_only=True).dtype
+        String
     """
 
 
@@ -397,21 +291,11 @@ class Boolean(DType):
     """Boolean type.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = [True, False, False, True]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> nw.from_native(ser_pd, series_only=True).dtype
-       Boolean
-       >>> nw.from_native(ser_pl, series_only=True).dtype
-       Boolean
-       >>> nw.from_native(ser_pa, series_only=True).dtype
-       Boolean
+        >>> import pyarrow as pa
+        >>> import narwhals as nw
+        >>> s_native = pa.chunked_array([[True, False, False, True]])
+        >>> nw.from_native(s_native, series_only=True).dtype
+        Boolean
     """
 
 
@@ -419,18 +303,12 @@ class Object(DType):
     """Data type for wrapping arbitrary Python objects.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> class Foo: ...
-       >>> ser_pd = pd.Series([Foo(), Foo()])
-       >>> ser_pl = pl.Series([Foo(), Foo()])
-
-       >>> nw.from_native(ser_pd, series_only=True).dtype
-       Object
-       >>> nw.from_native(ser_pl, series_only=True).dtype
-       Object
+        >>> import pandas as pd
+        >>> import narwhals as nw
+        >>> class Foo: ...
+        >>> s_native = pd.Series([Foo(), Foo()])
+        >>> nw.from_native(s_native, series_only=True).dtype
+        Object
     """
 
 
@@ -438,13 +316,11 @@ class Unknown(DType):
     """Type representing DataType values that could not be determined statically.
 
     Examples:
-       >>> import pandas as pd
-       >>> import narwhals as nw
-       >>> data = pd.period_range("2000-01", periods=4, freq="M")
-       >>> ser_pd = pd.Series(data)
-
-       >>> nw.from_native(ser_pd, series_only=True).dtype
-       Unknown
+        >>> import pandas as pd
+        >>> import narwhals as nw
+        >>> s_native = pd.Series(pd.period_range("2000-01", periods=4, freq="M"))
+        >>> nw.from_native(s_native, series_only=True).dtype
+        Unknown
     """
 
 
@@ -470,39 +346,20 @@ class Datetime(TemporalType, metaclass=_DatetimeMeta):
         Adapted from [Polars implementation](https://github.com/pola-rs/polars/blob/py-1.7.1/py-polars/polars/datatypes/classes.py#L398-L457)
 
     Examples:
-        >>> import pandas as pd
-        >>> import polars as pl
-        >>> import pyarrow as pa
-        >>> import pyarrow.compute as pc
-        >>> import narwhals as nw
         >>> from datetime import datetime, timedelta
-        >>> data = [datetime(2024, 12, 9) + timedelta(days=n) for n in range(5)]
-        >>> ser_pd = (
-        ...     pd.Series(data)
-        ...     .dt.tz_localize("Africa/Accra")
-        ...     .astype("datetime64[ms, Africa/Accra]")
-        ... )
-        >>> ser_pl = (
-        ...     pl.Series(data)
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> s_native = (
+        ...     pl.Series([datetime(2024, 12, 9) + timedelta(days=n) for n in range(5)])
         ...     .cast(pl.Datetime("ms"))
         ...     .dt.replace_time_zone("Africa/Accra")
         ... )
-        >>> ser_pa = pc.assume_timezone(
-        ...     pa.chunked_array([data], type=pa.timestamp("ms")), "Africa/Accra"
-        ... )
-
-        >>> nw.from_native(ser_pd, series_only=True).dtype
-        Datetime(time_unit='ms', time_zone='Africa/Accra')
-        >>> nw.from_native(ser_pl, series_only=True).dtype
-        Datetime(time_unit='ms', time_zone='Africa/Accra')
-        >>> nw.from_native(ser_pa, series_only=True).dtype
+        >>> nw.from_native(s_native, series_only=True).dtype
         Datetime(time_unit='ms', time_zone='Africa/Accra')
     """
 
     def __init__(
-        self: Self,
-        time_unit: TimeUnit = "us",
-        time_zone: str | timezone | None = None,
+        self, time_unit: TimeUnit = "us", time_zone: str | timezone | None = None
     ) -> None:
         if time_unit not in {"s", "ms", "us", "ns"}:
             msg = (
@@ -517,7 +374,7 @@ class Datetime(TemporalType, metaclass=_DatetimeMeta):
         self.time_unit: TimeUnit = time_unit
         self.time_zone: str | None = time_zone
 
-    def __eq__(self: Self, other: object) -> bool:
+    def __eq__(self, other: object) -> bool:
         # allow comparing object instances to class
         if type(other) is _DatetimeMeta:
             return True
@@ -526,10 +383,10 @@ class Datetime(TemporalType, metaclass=_DatetimeMeta):
         else:  # pragma: no cover
             return False
 
-    def __hash__(self: Self) -> int:  # pragma: no cover
+    def __hash__(self) -> int:  # pragma: no cover
         return hash((self.__class__, self.time_unit, self.time_zone))
 
-    def __repr__(self: Self) -> str:  # pragma: no cover
+    def __repr__(self) -> str:  # pragma: no cover
         class_name = self.__class__.__name__
         return f"{class_name}(time_unit={self.time_unit!r}, time_zone={self.time_zone!r})"
 
@@ -550,26 +407,18 @@ class Duration(TemporalType, metaclass=_DurationMeta):
         Adapted from [Polars implementation](https://github.com/pola-rs/polars/blob/py-1.7.1/py-polars/polars/datatypes/classes.py#L460-L502)
 
     Examples:
-        >>> import pandas as pd
-        >>> import polars as pl
+        >>> from datetime import timedelta
         >>> import pyarrow as pa
         >>> import narwhals as nw
-        >>> from datetime import timedelta
-        >>> data = [timedelta(seconds=d) for d in range(1, 4)]
-        >>> ser_pd = pd.Series(data).astype("timedelta64[ms]")
-        >>> ser_pl = pl.Series(data).cast(pl.Duration("ms"))
-        >>> ser_pa = pa.chunked_array([data], type=pa.duration("ms"))
-
-        >>> nw.from_native(ser_pd, series_only=True).dtype
-        Duration(time_unit='ms')
-        >>> nw.from_native(ser_pl, series_only=True).dtype
-        Duration(time_unit='ms')
-        >>> nw.from_native(ser_pa, series_only=True).dtype
+        >>> s_native = pa.chunked_array(
+        ...     [[timedelta(seconds=d) for d in range(1, 4)]], type=pa.duration("ms")
+        ... )
+        >>> nw.from_native(s_native, series_only=True).dtype
         Duration(time_unit='ms')
     """
 
-    def __init__(self: Self, time_unit: TimeUnit = "us") -> None:
-        if time_unit not in ("s", "ms", "us", "ns"):
+    def __init__(self, time_unit: TimeUnit = "us") -> None:
+        if time_unit not in {"s", "ms", "us", "ns"}:
             msg = (
                 "invalid `time_unit`"
                 f"\n\nExpected one of {{'ns','us','ms', 's'}}, got {time_unit!r}."
@@ -578,7 +427,7 @@ class Duration(TemporalType, metaclass=_DurationMeta):
 
         self.time_unit: TimeUnit = time_unit
 
-    def __eq__(self: Self, other: object) -> bool:
+    def __eq__(self, other: object) -> bool:
         # allow comparing object instances to class
         if type(other) is _DurationMeta:
             return True
@@ -587,10 +436,10 @@ class Duration(TemporalType, metaclass=_DurationMeta):
         else:  # pragma: no cover
             return False
 
-    def __hash__(self: Self) -> int:  # pragma: no cover
+    def __hash__(self) -> int:  # pragma: no cover
         return hash((self.__class__, self.time_unit))
 
-    def __repr__(self: Self) -> str:  # pragma: no cover
+    def __repr__(self) -> str:  # pragma: no cover
         class_name = self.__class__.__name__
         return f"{class_name}(time_unit={self.time_unit!r})"
 
@@ -599,39 +448,60 @@ class Categorical(DType):
     """A categorical encoding of a set of strings.
 
     Examples:
-       >>> import pandas as pd
-       >>> import polars as pl
-       >>> import pyarrow as pa
-       >>> import narwhals as nw
-       >>> data = ["beluga", "narwhal", "orca", "vaquita"]
-       >>> ser_pd = pd.Series(data)
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> nw.from_native(ser_pd, series_only=True).cast(nw.Categorical).dtype
-       Categorical
-       >>> nw.from_native(ser_pl, series_only=True).cast(nw.Categorical).dtype
-       Categorical
-       >>> nw.from_native(ser_pa, series_only=True).cast(nw.Categorical).dtype
-       Categorical
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> s_native = pl.Series(["beluga", "narwhal", "orca"])
+        >>> nw.from_native(s_native, series_only=True).cast(nw.Categorical).dtype
+        Categorical
     """
 
 
 class Enum(DType):
     """A fixed categorical encoding of a unique set of strings.
 
-    Polars has an Enum data type, while pandas and PyArrow do not.
+    Polars has an Enum data type. In pandas, ordered categories get mapped
+    to Enum. PyArrow has no Enum equivalent.
 
     Examples:
-       >>> import polars as pl
        >>> import narwhals as nw
-       >>> data = ["beluga", "narwhal", "orca", "vaquita"]
-       >>> ser_pl = pl.Series(data, dtype=pl.Enum(data))
-
-       >>> nw.from_native(ser_pl, series_only=True).dtype
-       Enum
+       >>> nw.Enum(["beluga", "narwhal", "orca"])
+       Enum(categories=['beluga', 'narwhal', 'orca'])
     """
 
+    def __init__(self, categories: Iterable[str] | type[enum.Enum]) -> None:
+        self._delayed_categories: _DeferredIterable[str] | None = None
+        self._cached_categories: tuple[str, ...] | None = None
+
+        if isinstance(categories, _DeferredIterable):
+            self._delayed_categories = categories
+        elif isinstance(categories, type) and issubclass(categories, enum.Enum):
+            self._cached_categories = tuple(member.value for member in categories)
+        else:
+            self._cached_categories = tuple(categories)
+
+    @property
+    def categories(self) -> tuple[str, ...]:
+        if cached := self._cached_categories:
+            return cached
+        elif delayed := self._delayed_categories:
+            self._cached_categories = delayed.to_tuple()
+            return self._cached_categories
+        else:  # pragma: no cover
+            msg = f"Internal structure of {type(self).__name__!r} is invalid."
+            raise TypeError(msg)
+
+    def __eq__(self, other: object) -> bool:
+        # allow comparing object instances to class
+        if type(other) is type:
+            return other is Enum
+        return isinstance(other, type(self)) and self.categories == other.categories
+
+    def __hash__(self) -> int:
+        return hash((self.__class__, tuple(self.categories)))
+
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}(categories={list(self.categories)!r})"
+
 
 class Field:
     """Definition of a single field within a `Struct` DataType.
@@ -639,22 +509,30 @@ class Field:
     Arguments:
         name: The name of the field within its parent `Struct`.
         dtype: The `DataType` of the field's values.
+
+    Examples:
+       >>> import pyarrow as pa
+       >>> import narwhals as nw
+       >>> data = [{"a": 1, "b": ["narwhal", "beluga"]}, {"a": 2, "b": ["orca"]}]
+       >>> ser_pa = pa.chunked_array([data])
+       >>> nw.from_native(ser_pa, series_only=True).dtype.fields
+       [Field('a', Int64), Field('b', List(String))]
     """
 
     name: str
-    dtype: type[DType] | DType
+    dtype: IntoDType
 
-    def __init__(self: Self, name: str, dtype: type[DType] | DType) -> None:
+    def __init__(self, name: str, dtype: IntoDType) -> None:
         self.name = name
         self.dtype = dtype
 
-    def __eq__(self: Self, other: Field) -> bool:  # type: ignore[override]
+    def __eq__(self, other: Field) -> bool:  # type: ignore[override]
         return (self.name == other.name) & (self.dtype == other.dtype)
 
-    def __hash__(self: Self) -> int:
+    def __hash__(self) -> int:
         return hash((self.name, self.dtype))
 
-    def __repr__(self: Self) -> str:
+    def __repr__(self) -> str:
         class_name = self.__class__.__name__
         return f"{class_name}({self.name!r}, {self.dtype})"
 
@@ -667,30 +545,24 @@ class Struct(NestedType):
             objects or a mapping of column names to data types.
 
     Examples:
-       >>> import polars as pl
        >>> import pyarrow as pa
        >>> import narwhals as nw
-       >>> data = [{"a": 1, "b": ["narwhal", "beluga"]}, {"a": 2, "b": ["orca"]}]
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> nw.from_native(ser_pl, series_only=True).dtype
-       Struct({'a': Int64, 'b': List(String)})
-       >>> nw.from_native(ser_pa, series_only=True).dtype
+       >>> s_native = pa.chunked_array(
+       ...     [[{"a": 1, "b": ["narwhal", "beluga"]}, {"a": 2, "b": ["orca"]}]]
+       ... )
+       >>> nw.from_native(s_native, series_only=True).dtype
        Struct({'a': Int64, 'b': List(String)})
     """
 
     fields: list[Field]
 
-    def __init__(
-        self: Self, fields: Sequence[Field] | Mapping[str, DType | type[DType]]
-    ) -> None:
+    def __init__(self, fields: Sequence[Field] | Mapping[str, IntoDType]) -> None:
         if isinstance(fields, Mapping):
             self.fields = list(starmap(Field, fields.items()))
         else:
             self.fields = list(fields)
 
-    def __eq__(self: Self, other: DType | type[DType]) -> bool:  # type: ignore[override]
+    def __eq__(self, other: DType | type[DType]) -> bool:  # type: ignore[override]
         # The comparison allows comparing objects to classes, and specific
         # inner types to those without (eg: inner=None). if one of the
         # arguments is not specific about its inner type we infer it
@@ -702,22 +574,22 @@ class Struct(NestedType):
         else:
             return False
 
-    def __hash__(self: Self) -> int:
+    def __hash__(self) -> int:
         return hash((self.__class__, tuple(self.fields)))
 
-    def __iter__(self: Self) -> Iterator[tuple[str, DType | type[DType]]]:
+    def __iter__(self) -> Iterator[tuple[str, IntoDType]]:  # pragma: no cover
         for fld in self.fields:
             yield fld.name, fld.dtype
 
-    def __reversed__(self: Self) -> Iterator[tuple[str, DType | type[DType]]]:
+    def __reversed__(self) -> Iterator[tuple[str, IntoDType]]:
         for fld in reversed(self.fields):
             yield fld.name, fld.dtype
 
-    def __repr__(self: Self) -> str:
+    def __repr__(self) -> str:
         class_name = self.__class__.__name__
         return f"{class_name}({dict(self)})"
 
-    def to_schema(self: Self) -> OrderedDict[str, DType | type[DType]]:
+    def to_schema(self) -> OrderedDict[str, IntoDType]:
         """Return Struct dtype as a schema dict.
 
         Returns:
@@ -731,28 +603,22 @@ class List(NestedType):
 
     Examples:
        >>> import pandas as pd
-       >>> import polars as pl
        >>> import pyarrow as pa
        >>> import narwhals as nw
-       >>> data = [["narwhal", "orca"], ["beluga", "vaquita"]]
-       >>> ser_pd = pd.Series(data, dtype=pd.ArrowDtype(pa.large_list(pa.large_string())))
-       >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
-
-       >>> nw.from_native(ser_pd, series_only=True).dtype
-       List(String)
-       >>> nw.from_native(ser_pl, series_only=True).dtype
-       List(String)
-       >>> nw.from_native(ser_pa, series_only=True).dtype
+       >>> s_native = pd.Series(
+       ...     [["narwhal", "orca"], ["beluga", "vaquita"]],
+       ...     dtype=pd.ArrowDtype(pa.large_list(pa.large_string())),
+       ... )
+       >>> nw.from_native(s_native, series_only=True).dtype
        List(String)
     """
 
-    inner: DType | type[DType]
+    inner: IntoDType
 
-    def __init__(self: Self, inner: DType | type[DType]) -> None:
+    def __init__(self, inner: IntoDType) -> None:
         self.inner = inner
 
-    def __eq__(self: Self, other: DType | type[DType]) -> bool:  # type: ignore[override]
+    def __eq__(self, other: DType | type[DType]) -> bool:  # type: ignore[override]
         # This equality check allows comparison of type classes and type instances.
         # If a parent type is not specific about its inner type, we infer it as equal:
         # > list[i64] == list[i64] -> True
@@ -767,10 +633,10 @@ class List(NestedType):
         else:
             return False
 
-    def __hash__(self: Self) -> int:
+    def __hash__(self) -> int:
         return hash((self.__class__, self.inner))
 
-    def __repr__(self: Self) -> str:
+    def __repr__(self) -> str:
         class_name = self.__class__.__name__
         return f"{class_name}({self.inner!r})"
 
@@ -783,30 +649,18 @@ class Array(NestedType):
         shape: The shape of the arrays.
 
     Examples:
-        >>> import pandas as pd
         >>> import polars as pl
-        >>> import pyarrow as pa
         >>> import narwhals as nw
-        >>> data = [[1, 2], [3, 4], [5, 6]]
-        >>> ser_pd = pd.Series(data, dtype=pd.ArrowDtype(pa.list_(pa.int32(), 2)))
-        >>> ser_pl = pl.Series(data, dtype=pl.Array(pl.Int32, 2))
-        >>> ser_pa = pa.chunked_array([data], type=pa.list_(pa.int32(), 2))
-
-        >>> nw.from_native(ser_pd, series_only=True).dtype
-        Array(Int32, shape=(2,))
-        >>> nw.from_native(ser_pl, series_only=True).dtype
-        Array(Int32, shape=(2,))
-        >>> nw.from_native(ser_pa, series_only=True).dtype
+        >>> s_native = pl.Series([[1, 2], [3, 4], [5, 6]], dtype=pl.Array(pl.Int32, 2))
+        >>> nw.from_native(s_native, series_only=True).dtype
         Array(Int32, shape=(2,))
     """
 
-    inner: DType | type[DType]
+    inner: IntoDType
     size: int
     shape: tuple[int, ...]
 
-    def __init__(
-        self: Self, inner: DType | type[DType], shape: int | tuple[int, ...]
-    ) -> None:
+    def __init__(self, inner: IntoDType, shape: int | tuple[int, ...]) -> None:
         inner_shape: tuple[int, ...] = inner.shape if isinstance(inner, Array) else ()
         if isinstance(shape, int):
             self.inner = inner
@@ -825,7 +679,7 @@ class Array(NestedType):
             msg = f"invalid input for shape: {shape!r}"
             raise TypeError(msg)
 
-    def __eq__(self: Self, other: DType | type[DType]) -> bool:  # type: ignore[override]
+    def __eq__(self, other: DType | type[DType]) -> bool:  # type: ignore[override]
         # This equality check allows comparison of type classes and type instances.
         # If a parent type is not specific about its inner type, we infer it as equal:
         # > array[i64] == array[i64] -> True
@@ -843,7 +697,7 @@ class Array(NestedType):
         else:
             return False
 
-    def __hash__(self: Self) -> int:
+    def __hash__(self) -> int:
         return hash((self.__class__, self.inner, self.shape))
 
     def __repr__(self) -> str:
@@ -860,20 +714,61 @@ class Date(TemporalType):
     """Data type representing a calendar date.
 
     Examples:
-       >>> import pandas as pd
+        >>> from datetime import date, timedelta
+        >>> import pyarrow as pa
+        >>> import narwhals as nw
+        >>> s_native = pa.chunked_array(
+        ...     [[date(2024, 12, 1) + timedelta(days=d) for d in range(4)]]
+        ... )
+        >>> nw.from_native(s_native, series_only=True).dtype
+        Date
+    """
+
+
+class Time(TemporalType):
+    """Data type representing the time of day.
+
+    Examples:
        >>> import polars as pl
        >>> import pyarrow as pa
        >>> import narwhals as nw
-       >>> from datetime import date, timedelta
-       >>> data = [date(2024, 12, 1) + timedelta(days=d) for d in range(4)]
-       >>> ser_pd = pd.Series(data, dtype="date32[pyarrow]")
+       >>> import duckdb
+       >>> from datetime import time
+       >>> data = [time(9, 0), time(9, 1, 10), time(9, 2)]
        >>> ser_pl = pl.Series(data)
-       >>> ser_pa = pa.chunked_array([data])
+       >>> ser_pa = pa.chunked_array([pa.array(data, type=pa.time64("ns"))])
+       >>> rel = duckdb.sql(
+       ...     " SELECT * FROM (VALUES (TIME '12:00:00'), (TIME '14:30:15')) df(t)"
+       ... )
 
-       >>> nw.from_native(ser_pd, series_only=True).dtype
-       Date
        >>> nw.from_native(ser_pl, series_only=True).dtype
-       Date
+       Time
        >>> nw.from_native(ser_pa, series_only=True).dtype
-       Date
+       Time
+       >>> nw.from_native(rel).collect_schema()["t"]
+       Time
+    """
+
+
+class Binary(DType):
+    """Binary type.
+
+    Examples:
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> import pyarrow as pa
+        >>> import duckdb
+        >>> data = [b"test1", b"test2"]
+        >>> ser_pl = pl.Series(data, dtype=pl.Binary)
+        >>> ser_pa = pa.chunked_array([pa.array(data, type=pa.binary())])
+        >>> rel = duckdb.sql(
+        ...     "SELECT * FROM (VALUES (BLOB 'test1'), (BLOB 'test2')) AS df(t)"
+        ... )
+
+        >>> nw.from_native(ser_pl, series_only=True).dtype
+        Binary
+        >>> nw.from_native(ser_pa, series_only=True).dtype
+        Binary
+        >>> nw.from_native(rel).collect_schema()["t"]
+        Binary
     """
