@@ -1,24 +1,15 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-from typing import Any
 from typing import Generic
-from typing import TypeVar
 
-if TYPE_CHECKING:
-    from typing_extensions import Self
-
-    from narwhals.series import Series
-
-
-SeriesT = TypeVar("SeriesT", bound="Series[Any]")
+from narwhals.typing import SeriesT
 
 
 class SeriesListNamespace(Generic[SeriesT]):
-    def __init__(self: Self, series: SeriesT) -> None:
+    def __init__(self, series: SeriesT) -> None:
         self._narwhals_series = series
 
-    def len(self: Self) -> SeriesT:
+    def len(self) -> SeriesT:
         """Return the number of elements in each list.
 
         Null values count towards the total.
@@ -42,6 +33,6 @@ class SeriesListNamespace(Generic[SeriesT]):
               ]
             ]
         """
-        return self._narwhals_series._from_compliant_series(
+        return self._narwhals_series._with_compliant(
             self._narwhals_series._compliant_series.list.len()
         )
